from .rope import iter_rope

class IoBlob:
    def __init__(self, file, start=0, stop=None):
        self._file = file
        self._start = start

        if stop is None:
            file.seek(0, 2)
            self._len = file.tell()
        else:
            self._len = stop - start

    def __len__(self):
        return self._len

    def __getitem__(self, key):
        if not isinstance(key, slice):
            key = slice(key, key + 1)
        start, stop, step = key.indices(self._len)
        if step != 1:
            raise IndexError('strides are not supported')

        return IoBlob(self._file, self._start + start, self._start + stop)

    def __iterrope__(self):
        if self._len == 0:
            return

        self._file.seek(self._start, 0)

        read = 0
        while read < self._len:
            r = self._file.read(self._len - read)
            if not r:
                raise IOError('reached eof prematurely')
            yield r
            read += len(r)

    def __bytes__(self):
        return b''.join(iter_rope(self))



from .bytes_io import IoBlob
from .rope import iter_rope

def wrap_file(file, mode='rb'):
    if 'b' in mode:
        return IoBlob(file)

    raise RuntimeError('unsupported mode (only binary is allowed)')

def open(name, mode):
    file = open(name, mode)
    return wrap_file(file, mode)

def write(fout, rope):
    for chunk in iter_rope(rope):
        fout.write(chunk)



#!/usr/bin/env python
# coding: utf-8

from setuptools import setup

setup(
    name='grope',
    version='0.1',

    description='Generalized rope data structure',
    author='Martin Vejnár',
    author_email='vejnar.martin@gmail.com',
    url='https://github.com/avakar/grope',
    license='MIT',

    packages=['grope'],
    )



class _Rope:
    def __init__(self, children):
        self._children = children
        self._len = sum(len(c) for c in children)

    def __len__(self):
        return self._len

    def __getitem__(self, key):
        if not isinstance(key, slice):
            key = slice(key, key + 1)
        start, stop, step = key.indices(self._len)
        if step != 1:
            raise RuntimeError('strides are not supported')

        size = stop - start

        parts = []
        for c in self._children:
            if len(c) <= start:
                start -= len(c)
                continue

            if start + size <= len(c):
                parts.append(c[start:start + size])
                break

            parts.append(c[start:])
            size -= len(c) - start
            start = 0

        return rope(*parts)

    def __iterrope__(self):
        for c in self._children:
            for chunk in iter_rope(c):
                yield chunk

    def __bytes__(self):
        return b''.join(iter_rope(self))

    def __str__(self):
        return ''.join(iter_rope(self))

def iter_rope(rope):
    iterrope = getattr(rope, '__iterrope__', None)
    if iterrope is None:
        yield rope
    else:
        for chunk in iterrope():
            yield chunk

def rope(*ropables):
    # TODO: optimize, in particular, actually implement rope data structure
    return _Rope(ropables)
